@@ -7,8 +7,8 @@
 #' For more details, see the section below.
 #'
 #' @section Data format:
-#' An input \code{image} can have one of three following types: 2-dimensional matrix representaing \emph{grayscale} image, 3-dimensional array
-#' for \emph{color} image whose 3rd dimension has size of 3, or \pkg{cimg} object represented as 4-d array.
+#' An input \code{data} can be either (1) 2-dimensional matrix representaing \emph{grayscale} image, or (2) 3-dimensional array
+#' for \emph{color} image.
 #'
 #' @section Algorithms for TV-L1 problem:
 #' The cost function for TV-L2 problem is
@@ -26,31 +26,35 @@
 #' }
 #'
 #'
-#' @param image standard 2d or 3d array, or \code{cimg} object.
+#' @param data standard 2d or 3d array.
 #' @param lambda regularization parameter (positive real number).
 #' @param niter  total number of iterations.
 #' @param method indicating problem and algorithm combination.
+#' @param normalize a logical; \code{TRUE} to make the range in \eqn{[0,1]}, or \code{FALSE} otherwise.
 #'
-#' @return denoised image of same data type as input \code{image}.
+#' @return denoised array as same size of \code{data}.
 #'
 #' @examples
-#' ## Load data from 'imager' library and rescale it
-#' library(imager)
-#' x = imager::imresize(imager::boats, scale=0.5)
+#' \dontrun{
+#' ## Load grey-scale 'lena' data
+#' library(filling)
+#' data(lena128)
 #'
 #' ## Add white noise
-#' xnoised = x + imager::imnoise(dim=dim(x), sd=0.1); plot(xnoised)
+#' sinfo   <- dim(lena128)   # get the size information
+#' xnoised <- lena128 + array(rnorm(sinfo, sd=10), sinfo)
 #'
 #' ## apply denoising models
-#' xproc1 = denoise2(xnoised, method="TVL2.FiniteDifference")
-#' xproc2 = denoise2(xnoised, method="TVL1.PrimalDual")
+#' xproc1 <- denoise2(xnoised, lambda=10, method="TVL2.FiniteDifference")
+#' xproc2 <- denoise2(xnoised, lambda=10, method="TVL1.PrimalDual")
 #'
 #' ## compare
-#' par(mfrow=c(2,2))
-#' plot(x, main="original")
-#' plot(xnoised, main="noised")
-#' plot(xproc1, main="TVL2.FiniteDifference")
-#' plot(xproc2, main="TVL1.PrimalDual")
+#' par(mfrow=c(2,2), pty="s")
+#' image(lena128, main="original")
+#' image(xnoised, main="noised")
+#' image(xproc1, main="TVL2.FiniteDifference")
+#' image(xproc2, main="TVL1.PrimalDual")
+#' }
 #'
 #' @references
 #' \insertRef{rudin_nonlinear_1992}{tvR}
@@ -58,7 +62,8 @@
 #' \insertRef{chambolle_first-order_2011}{tvR}
 #'
 #' @export
-denoise2 <- function(image, lambda=1.0, niter=100, method=c("TVL1.PrimalDual","TVL2.PrimalDual","TVL2.FiniteDifference")){
+denoise2 <- function(data, lambda=1.0, niter=100, method=c("TVL1.PrimalDual","TVL2.PrimalDual","TVL2.FiniteDifference"),
+                     normalize=FALSE){
   ## Check Data, Lambda, niter
   ##    For image data as cimg, it's fine.
   if (!check_lambda(lambda)){
@@ -67,15 +72,12 @@ denoise2 <- function(image, lambda=1.0, niter=100, method=c("TVL1.PrimalDual","T
   if (!check_niter(niter)){
     stop("* denoise2 : 'niter' should be a positive integer larger than 1.")
   }
-  checkcimg = is.cimg(image)
-  if (!checkcimg){
-    if (!check_data_image(image)){
-      stop("* denoise2 : an 'image' array should be 2d or 3d with 3rd dimension having size of 3.")
-    }
-    oldw  = getOption("warn")
-    options(warn = -1)
-    image = as.cimg(image)
-    options(warn=oldw)
+  if (!is.array(data)){
+    stop("* denoise2 : 'data' should be either a matrix or an array.")
+  }
+  ndim = length(dim(data))
+  if (!((ndim==2)||(ndim==3))){
+    stop("* denoise2 : 'data' should be either 2d- or 3d-array.")
   }
 
   ## Method Argument
@@ -87,9 +89,9 @@ denoise2 <- function(image, lambda=1.0, niter=100, method=c("TVL1.PrimalDual","T
 
   ## Main Computation Part
   output = switch(method,
-                  TVL2.PrimalDual       = denoise2.TVL2.PrimalDual(image, 1.0/lambda, niter, checkcimg),
-                  TVL2.FiniteDifference = denoise2.TVL2.FiniteDifference(image, 1.0/lambda, niter, checkcimg),
-                  TVL1.PrimalDual       = denoise2.TVL1.PrimalDual(image, 1.0/lambda, niter, checkcimg)
+                  TVL2.PrimalDual       = denoise2.TVL2.PrimalDual(data, 1.0/lambda, niter, normalize),
+                  TVL2.FiniteDifference = denoise2.TVL2.FiniteDifference(data, 1.0/lambda, niter, normalize),
+                  TVL1.PrimalDual       = denoise2.TVL1.PrimalDual(data, 1.0/lambda, niter, normalize)
                   )
 
   ## return output
